from django.shortcuts import render

from products.models import Product
from purchases.models import Purchase, PurchaseOrder
from suppliers.models import Supplier
from django.contrib.auth.decorators import login_required

@login_required
def purchase(request): 
    products = Product.objects.all()
    suppliers = Supplier.objects.all()
    purchaseCart = Purchase.objects.filter(purchase_order = None)
 
    context = {
        'products': products,
        'suppliers': suppliers,
        'purchaseCart':purchaseCart
    }
    
    return render(request, "sales/purchase/index.html", context)
@login_required
def purchases(request):
    return render(request, "sales/purchase/purchases.html")

from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt 
import json

@csrf_exempt
@login_required
def create_purchase_order(request):
    if request.method == 'POST':
        data = json.loads(request.body)
        supplier_id = data['supplier']
        delivery_date = data['delivery_date']
        purchases_data = data['purchases']

        try:
            supplier = Supplier.objects.get(id=supplier_id)
            user = request.user  # Assuming the user is logged in

            # Create PurchaseOrder
            purchase_order = PurchaseOrder.objects.create(
                supplier=supplier,
                ordered_by=user,
                delivery_date=delivery_date
            )

            # Create Purchases
            for purchase_data in purchases_data:
                product = Product.objects.get(id=purchase_data['product_id'])
                Purchase.objects.create(
                    purchase_order=purchase_order,
                    product=product,
                    quantity=purchase_data['quantity'],
                    price_per_unit=purchase_data['price_per_unit']
                )

            return JsonResponse({'success': True})

        except Exception as e:
            return JsonResponse({'success': False, 'error': str(e)})
    else:
        return JsonResponse({'success': False, 'error': 'Invalid request method'})


 
from customers.models import Customer
from account.models import Account
from config.apipermissions import AllowAllPermissions 
from django.core.files import File
from io import BytesIO
from rest_framework import status
from rest_framework.response import Response
from rest_framework.decorators import api_view, permission_classes
from purchases.models import  Purchase 

@api_view(['POST'])
@permission_classes([AllowAllPermissions])  
def create_puchase(request):
    if request.method == 'POST':
        try:
            product_id = request.data.get('product_id')
            quontity = request.data.get('quantity')
            price_per_unit = request.data.get('price_per_unit')

            Purchase.objects.create(purchase_order=None, product=Product.objects.filter(product_id=product_id).first(),quantity=quontity,price_per_unit=price_per_unit) 
 
            return Response({'message': 'Quotation created successfully!', 'success': True}, status=status.HTTP_201_CREATED)

        except Exception as e:
            print(f"Error creating quotation: {e}")
            return Response({'error': 'An error occurred.'}, status=status.HTTP_400_BAD_REQUEST)

    else:
        return Response({'error': 'Invalid request method. Use POST.'}, status=status.HTTP_405_METHOD_NOT_ALLOWED)
    
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt 
import json
from products.models import Product
from purchases.models import Purchase, PurchaseOrder
from suppliers.models import Supplier

@api_view(['POST'])
@permission_classes([AllowAllPermissions])  
def create_purchase_order(request):
    if request.method == 'POST':
        data = json.loads(request.body)
        supplier_id = data['supplier']
        delivery_date = data['delivery_date']

        try:
            # Fetch the supplier
            supplier = Supplier.objects.get(supplier_id=supplier_id) 

            # Check for purchases without an order associated
            purchases_without_order = Purchase.objects.filter(purchase_order=None)

            if not purchases_without_order.exists():
                return JsonResponse({
                    'success': False,
                    'error': 'No purchases available to create a purchase order.'
                })

            # Create the purchase order
            purchase_order = PurchaseOrder.objects.create(
                supplier=supplier, 
                delivery_date=delivery_date
            )
            print(purchase_order)

            # Associate purchases with the newly created order
            for purchase in purchases_without_order:
                purchase.purchase_order = purchase_order
                purchase.save()
 
            # Calculate total cost
            purchase_order.calculate_total_cost()

            return JsonResponse({
                'success': True,
                'message': 'Purchase order created successfully!'
            })

        except Supplier.DoesNotExist:
            return JsonResponse({
                'success': False,
                'error': 'Supplier not found.'
            })
        except Exception as e:
            return JsonResponse({
                'success': False,
                'error': str(e)
            })
    else:
        return JsonResponse({
            'success': False,
            'error': 'Invalid request method.'
        })

from django.http import JsonResponse
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny  # Make sure to use the correct permissions
from .models import PurchaseOrder, Purchase  # Import necessary models
import json

from django.http import JsonResponse
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny
from .models import Purchase, PurchaseOrder
import json

@api_view(['PUT'])
@permission_classes([AllowAny])
def update_purchase_order(request, pk):
    try:
        # Get the purchase order using the provided primary key
        order = Purchase.objects.get(id=pk)

        # Load the incoming JSON data from the request body
        data = json.loads(request.body)
        print(data)
# price_per_unit = models.DecimalField(max_digits=10, decimal_places=2)
#     total_cost = models.DecimalField(max_digits=10, decimal_places=2, editable=False)
        # Ensure quantity is an integer and price_per_unit is a decimal
        quantity = int(data.get('quantity', order.quantity))  # Convert to int
        price_per_unit = float(data.get('price', order.price_per_unit))  # Convert to float first

        # Update fields
        order.total_cost = quantity * price_per_unit
        order.quantity = quantity
        order.price_per_unit = price_per_unit


        # Save the updated order
        order.save()

        # Return a success response
        return JsonResponse({'success': True, 'message': 'Purchase order updated successfully.'})

    except Purchase.DoesNotExist:
        return JsonResponse({'success': False, 'error': 'Order not found.'}, status=404)
    except ValueError as ve:
        return JsonResponse({'success': False, 'error': str(ve)}, status=400)
    except Exception as e:
        return JsonResponse({'success': False, 'error': str(e)}, status=500)



@api_view(['DELETE'])
@permission_classes([AllowAllPermissions])  
def delete_purchase_order(request, pk):
    try:
        order = Purchase.objects.get(id=pk)
        order.delete()
        return JsonResponse({'success': True})
    except Purchase.DoesNotExist:
        return JsonResponse({'success': False, 'error': 'Order not found.'})
    except Exception as e:
        return JsonResponse({'success': False, 'error': str(e)})
    

from rest_framework import serializers
from .models import PurchaseOrder, Purchase

class PurchaseSerializer(serializers.ModelSerializer):
    product = serializers.CharField(source='product.product_name', read_only=True)
    class Meta:
        model = Purchase
        fields = ['id', 'product', 'quantity','Q_received', 'price_per_unit', 'total_cost', 'status', 'created_at', 'updated_at']

from rest_framework import serializers
from .models import PurchaseOrder, Supplier, Purchase

# Serializer for the Supplier model
class SupplierSerializer(serializers.ModelSerializer):
    class Meta:
        model = Supplier
        fields = ['supplier_name','phone']  # Add all relevant fields
 
# Serializer for the PurchaseOrder model
class PurchaseOrderSerializer(serializers.ModelSerializer):
    purchases = PurchaseSerializer(many=True, read_only=True)  # Fetching related purchases
    supplier = SupplierSerializer(read_only=True)  # Using SupplierSerializer to fetch all supplier info

    class Meta:
        model = PurchaseOrder
        fields = ['order_id', 'supplier', 'order_date', 'delivery_date', 'status', 'total_cost', 'created_at', 'updated_at', 'purchases']



from django.http import JsonResponse
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny
from .models import PurchaseOrder 

@api_view(['GET'])
@permission_classes([AllowAny])
def list_purchase_orders(request):
    """
    List all Purchase Orders
    """
    purchase_orders = PurchaseOrder.objects.order_by("-order_id")
    serializer = PurchaseOrderSerializer(purchase_orders, many=True)
    return JsonResponse({'purchase_orders': serializer.data})

@api_view(['GET'])
@permission_classes([AllowAny])
def get_purchase_order_details(request, pk):
    """
    Get details of a specific Purchase Order
    """
    try:
        purchase_order = PurchaseOrder.objects.get(order_id=pk)
        serializer = PurchaseOrderSerializer(purchase_order)
        return JsonResponse({'purchase_order': serializer.data})
    except PurchaseOrder.DoesNotExist:
        return JsonResponse({'success': False, 'error': 'Purchase Order not found.'}, status=404)

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status
from .models import PurchaseOrder, Purchase

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status
from .models import PurchaseOrder, Purchase

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status
from .models import PurchaseOrder, Purchase

@api_view(['POST'])
def receive_all_purchases(request, order_id):
    try:
        print(f"Received request to receive all purchases for order ID: {order_id}")

        # Use filter instead of get to avoid exceptions and provide better logging
        purchase_order = PurchaseOrder.objects.filter(order_id=order_id).first()
        if not purchase_order:
            print(f"Purchase order with ID {order_id} does not exist.")
            return Response({'error': 'Purchase order not found.'}, status=status.HTTP_404_NOT_FOUND)

        print(f"Found purchase order: {purchase_order}")
        order = PurchaseOrder.objects.get(order_id=order_id)
        # Use Purchase.objects.filter to get pending purchases for the order
        purchases = Purchase.objects.filter(purchase_order=order, status='Pending')
        print(f"Pending purchases: {purchases.count()} found.")

        if not purchases.exists():
            return Response({'message': 'No pending purchases to receive.'}, status=status.HTTP_200_OK)

        for purchase in purchases:
            print(f"Processing purchase: {purchase}")
            # Update the product stock and purchase status
            purchase.product.quantity_in_stock += purchase.quantity  # Deduct quantity from product stock
            purchase.product.save()  # Save the product with updated stock
            purchase.status = 'Completed'  # Update status to Completed
            purchase.save()  # Save the purchase with updated status
            print(f"Updated purchase: {purchase} with status {purchase.status}")

        purchase_order.calculate_total_cost()  # Update total cost after receiving items
        print(f"Total cost updated for order: {purchase_order.total_cost}")

        # Update purchase order status
        if Purchase.objects.filter(purchase_order=purchase_order, status='Pending').exists():
            purchase_order.status = 'Partially Received'
        else:
            purchase_order.status = 'Received'
        purchase_order.save()
        print(f"Updated purchase order status: {purchase_order.status}")

        return Response({'message': 'All items received successfully!'}, status=status.HTTP_200_OK)

    except Exception as e:
        print(f"Error occurred: {str(e)}")
        return Response({'error': str(e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

@api_view(['POST'])
def receive_individual_purchase(request, purchase_id):
    try:
        print(f"Received request to receive individual purchase with ID: {purchase_id}")

        purchase = Purchase.objects.filter(id=purchase_id).first()
        if not purchase:
            print(f"Purchase with ID {purchase_id} does not exist.")
            return Response({'error': 'Purchase not found.'}, status=status.HTTP_404_NOT_FOUND)

        print(f"Found purchase: {purchase}")

        if purchase.status == 'Pending':
            # Retrieve the received quantity from the request body
            received_quantity = request.data.get('received_quantity', 0)  # Assuming it's sent in the body

            # Update the product stock and purchase status
            purchase.product.quantity_in_stock += int(received_quantity)  # Add quantity to product stock
            purchase.product.save()  # Save the product with updated stock

            # Update the Q_received field
            purchase.Q_received += int(received_quantity)
            purchase.status = 'Completed'  # Update status to Completed
            purchase.total_cost = purchase.Q_received * purchase.price_per_unit
            purchase.save()  # Save the purchase with updated status
            print(f"Updated purchase: {purchase} with status {purchase.status} and Q_received {purchase.Q_received}")

            # Update the total cost of the related purchase order
            purchase.purchase_order.calculate_total_cost()
            print(f"Total cost updated for order: {purchase.purchase_order.total_cost}")

            # Update purchase order status
            purchase_order = purchase.purchase_order
            if Purchase.objects.filter(purchase_order=purchase_order, status='Pending').exists():
                purchase_order.status = 'Partially Received'
            else:
                purchase_order.status = 'Received'   
            purchase_order.save()
            print(f"Updated purchase order status: {purchase_order.status}")

            return Response({'message': 'Item received successfully!'}, status=status.HTTP_200_OK)
        else:
            print(f"Purchase {purchase_id} is already received or not pending.")
            return Response({'error': 'Item has already been received or is not pending.'}, status=status.HTTP_400_BAD_REQUEST)

    except Exception as e:
        print(f"Error occurred: {str(e)}")
        return Response({'error': str(e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
